@@ -45,10 +45,10 @@ pop.predict <- function(end.year=2100, start.year=1950, present.year=2010, wpp.y
 			stop('Prediction in ', outdir,
 			' already exists.\nSet replace.output=TRUE if you want to overwrite existing projections.')
 		unlink(outdir, recursive=TRUE)
-		.remove.cache.file(output.dir)
+		.remove.cache.file(outdir)
 	} else pop.cleanup.cache(pred)
 	if(!is.null(countries) && is.na(countries[1])) { # all countries that are not included in the existing prediction
-		all.countries <- unique(inp$POPm0[,'country_code'])
+		all.countries <- intersect(unique(inp$POPm0[,'country_code']), UNcountries())
 		country.codes <- if(!prediction.exist) all.countries
 						else all.countries[!is.element(all.countries, pred$countries[,'code'])]
 	} else {
@@ -64,7 +64,7 @@ pop.predict <- function(end.year=2100, start.year=1950, present.year=2010, wpp.y
 			}
 			country.codes <- as.integer(countries)
 		} else
-			country.codes <- unique(inp$POPm0[,'country_code'])
+			country.codes <- intersect(unique(inp$POPm0[,'country_code']), UNcountries())
 	}
 	do.pop.predict(country.codes, inp, outdir, nr.traj, ages, pred=if(prediction.exist) pred else NULL,
 					keep.vital.events=keep.vital.events, function.inputs=inputs, verbose=verbose)
@@ -83,7 +83,6 @@ do.pop.predict <- function(country.codes, inp, outdir, nr.traj, ages, pred=NULL,
 		}
 		countries.idx[icountry] <- country.idx
 	}
-
 	if(length(not.valid.countries.idx) > 0) {
 		warning('Countries ', paste(country.codes[not.valid.countries.idx], collapse=', '), 
 					' not found in the UNlocations dataset.')
@@ -97,20 +96,21 @@ do.pop.predict <- function(country.codes, inp, outdir, nr.traj, ages, pred=NULL,
 	if(!file.exists(outdir)) 
 		dir.create(outdir, recursive=TRUE)
 	present.and.proj.years <- c(inp$estim.years[nest], inp$proj.years)
+	present.and.proj.years.pop <- present.and.proj.years + 2
 	prediction.file <- file.path(outdir, 'prediction.rda')	
 	quantiles.to.keep <- get.quantiles.to.keep()
 	nquant <- length(quantiles.to.keep)
 	PIs_cqp <- quantM <- quantF <- array(NA, c(ncountries, nquant, nr_project+1),
-						dimnames=list(country.codes, quantiles.to.keep, present.and.proj.years))
+						dimnames=list(country.codes, quantiles.to.keep, present.and.proj.years.pop))
 	quantMage <- quantFage <- quantPropMage <- quantPropFage <- array(NA, c(ncountries, nages, nquant, nr_project+1),
-						dimnames=list(country.codes, ages, quantiles.to.keep, present.and.proj.years))
+						dimnames=list(country.codes, ages, quantiles.to.keep, present.and.proj.years.pop))
 	mean_sd <- mean_sdM <- mean_sdF <- array(NA, c(ncountries, 2, nr_project+1), 
-						dimnames=list(country.codes, c('mean', 'sd'), present.and.proj.years))
+						dimnames=list(country.codes, c('mean', 'sd'), present.and.proj.years.pop))
 	mx.ages <- c(0,1,ages[2:nages])
 	status.for.gui <- paste('out of', ncountries, 'countries.')
 	gui.options <- list()
 	inp.to.save <- list()
-	# remove big items redundant items from inputs to be saved
+	# remove big or redundant items from inputs to be saved
 	for(item in ls(inp)[!grepl('^migMpred$|^migFpred$|^TFRpred$|^e0Fpred$|^e0Mpred$|^estim.years$|^proj.years$|^wpp.years$', ls(inp))]) 
 		inp.to.save[[item]] <- get(item, inp)
 	for(cidx in 1:ncountries) {
@@ -129,14 +129,14 @@ do.pop.predict <- function(country.codes, inp, outdir, nr.traj, ages, pred=NULL,
 		npred <- min(nrow(inpc$TFRpred), nr_project)
 		npredplus1 <- npred+1
 		totp <- matrix(NA, nrow=npredplus1, ncol=nr.traj, 
-					dimnames=list(present.and.proj.years, NULL))
+					dimnames=list(present.and.proj.years.pop, NULL))
 		totpm <- totpf <- array(NA, dim=c(27, npredplus1, nr.traj), 
-							dimnames=list(ages, present.and.proj.years, NULL))
+							dimnames=list(ages, present.and.proj.years.pop, NULL))
 		nvariants <- nrow(inpc$TFRhalfchild)
 		totp.hch <- matrix(NA, nrow=npredplus1, ncol=nvariants,
-					dimnames=list(present.and.proj.years, NULL))
+					dimnames=list(present.and.proj.years.pop, NULL))
 		totpm.hch <- totpf.hch <- array(NA, dim=c(27, npredplus1, nvariants), 
-					dimnames=list(ages, present.and.proj.years, NULL))
+					dimnames=list(ages, present.and.proj.years.pop, NULL))
 		if(keep.vital.events) {
 			btm <- btf <- array(0, dim=c(7, npredplus1, nr.traj), dimnames=list(NULL, present.and.proj.years, NULL))
 			deathsm <- deathsf <- array(0, dim=c(27, npredplus1, nr.traj), dimnames=list(ages, present.and.proj.years, NULL))
@@ -228,13 +228,13 @@ do.pop.predict <- function(country.codes, inp, outdir, nr.traj, ages, pred=NULL,
 			}
 		}
 		save(totp, totpm, totpf, totp.hch, totpm.hch, totpf.hch,
-			 file = file.path(outdir, paste('totpop_country', country, '.rda', sep='')))
+			 file = file.path(outdir, paste0('totpop_country', country, '.rda')))
 		if(keep.vital.events) 
 			save(btm, btf, deathsm, deathsf, asfert, mxm, mxf, migm, migf,
 				btm.hch, btf.hch, deathsm.hch, deathsf.hch, asfert.hch, 
 				mxm.hch, mxf.hch, 
 				observed,
-					file=file.path(outdir, paste('vital_events_country', country, '.rda', sep='')))
+					file=file.path(outdir, paste0('vital_events_country', country, '.rda')))
 		PIs_cqp[cidx,,] = apply(totp, 1, quantile, quantiles.to.keep, na.rm = TRUE)
 		mean_sd[cidx,1,] <- apply(totp, 1, mean, na.rm = TRUE)
 		mean_sd[cidx,2,] = apply(totp, 1, sd, na.rm = TRUE)
@@ -280,8 +280,8 @@ do.pop.predict <- function(country.codes, inp, outdir, nr.traj, ages, pred=NULL,
                				quantilesPropMage = quantPropMage[c(),,,,drop=FALSE], 
                				quantilesPropFage = quantPropFage[c(),,,,drop=FALSE],
                				estim.years=inp$estim.years, 
-               				proj.years=c(inp$estim.years[length(inp$estim.years)], 
-               									inp$proj.years), # includes present period
+               				proj.years=present.and.proj.years, # includes present period (middle of periods)
+               				proj.years.pop=present.and.proj.years.pop, # end of periods
                				wpp.year = inp$wpp.year,
 			   				inputs = inp.to.save, # save as list because environment takes much more space
 			   				function.inputs=function.inputs,
@@ -340,34 +340,27 @@ read.bayesPop.file <- function(file)
 
 load.inputs <- function(inputs, start.year, present.year, end.year, wpp.year, verbose=FALSE) {
 	observed <- list()
+	pop.ini.matrix <- pop.ini <- list(M=NULL, F=NULL)
 	# Get initial population counts
-	if(is.null(inputs$popM)) 
-		POPm0 <- load.wpp.dataset('popM', wpp.year)	
-	else POPm0 <- read.pop.file(inputs$popM)
-	num.columns <- grep('^[0-9]{4}$', colnames(POPm0), value=TRUE) # values of year-columns
-	if(!is.element(as.character(present.year), colnames(POPm0))) {
-		stop('Wrong present.year. ', present.year, ' not available in the popM file.\nAvailable years: ',
+	for(sex in c('M', 'F')) {
+		dataset.name <- paste0('pop', sex)
+		if(is.null(inputs[[dataset.name]])) 
+			POP0 <- load.wpp.dataset(dataset.name, wpp.year)	
+		else POP0 <- read.pop.file(inputs[[dataset.name]])
+		num.columns <- grep('^[0-9]{4}$', colnames(POP0), value=TRUE) # values of year-columns
+		if(!is.element(as.character(present.year), colnames(POP0))) {
+			stop('Wrong present.year. ', present.year, ' not available in the ', dataset.name, ' file.\nAvailable years: ',
 				paste(num.columns, collapse=', '))
+		}
+		num.columns <- num.columns[which(as.integer(num.columns)<= present.year)]
+		pop.ini.matrix[[sex]] <- POP0[,num.columns]
+		dimnames(pop.ini.matrix[[sex]]) <- list(paste(POP0[,'country_code'], POP0[,'age'], sep='_'), 
+									as.character(as.integer(num.columns)))
+		pop.ini[[sex]] <- POP0[,c('country_code', 'age', present.year)]
 	}
-	num.columns <- num.columns[which(as.integer(num.columns)<= present.year)]
-	popm.matrix <- POPm0[,num.columns]
-	dimnames(popm.matrix) <- list(paste(POPm0[,'country_code'], POPm0[,'age'], sep='_'), 
-									as.character(as.integer(num.columns)-2))
-	POPm0 <- POPm0[,c('country_code', 'age', present.year)]
-	
-	if(is.null(inputs$popF)) 
-		POPf0 <- load.wpp.dataset('popF', wpp.year)
-	else POPf0 <- read.pop.file(inputs$popF)
-	num.columns <- grep('^[0-9]{4}$', colnames(POPf0), value=TRUE) # values of year-columns
-	if(!is.element(as.character(present.year), colnames(POPf0))) {
-		stop('Wrong present.year. ', present.year, ' not available in the popF file.\nAvailable years: ',
-				paste(num.columns, collapse=', '))
-	}
-	num.columns <- num.columns[which(as.integer(num.columns)<= present.year)]
-	popf.matrix <- POPf0[, num.columns]
-	dimnames(popf.matrix) <- list(paste(POPf0[,'country_code'], POPf0[,'age'], sep='_'), 
-									as.character(as.integer(num.columns)-2))
-	POPf0 <- POPf0[,c('country_code', 'age', present.year)]
+	POPm0 <- pop.ini[['M']]
+	POPf0 <- pop.ini[['F']]
+
 	# Get death rates
 	if(is.null(inputs$mxM)) 
 		MXm <- load.wpp.dataset('mxM', wpp.year)
@@ -449,7 +442,7 @@ load.inputs <- function(inputs, start.year, present.year, end.year, wpp.year, ve
 			stop('File ', file.name, ' does not exist.')
 		# comma separated trajectories file
 		var.name <- paste0('mig',sex, 'pred')
-		if(verbose) cat('\nLoading ', file.name, '\n')
+		if(verbose) cat('\nLoading ', file.name)
 		migpred.raw <- read.csv(file=file.name, comment.char='#', check.names=FALSE)
 		migpred <- migpred.raw[,c('LocID', 'Year', 'Trajectory', 'Age', 'Migration')]
 		colnames(migpred) <- c('country_code', 'year', 'trajectory', 'age', 'value')
@@ -468,7 +461,7 @@ load.inputs <- function(inputs, start.year, present.year, end.year, wpp.year, ve
 				stop('File ', file.name, 
 					' does not exist.\nSet e0F.sim.dir, e0F.file or change WPP year.')
 		 	# comma separated trajectories file
-		 	if(verbose) cat('\nLoading ', file.name, '\n')
+		 	if(verbose) cat('\nLoading ', file.name)
 			e0Fpred <- read.csv(file=file.name, comment.char='#', check.names=FALSE)
 			e0Fpred <- e0Fpred[,c('LocID', 'Year', 'Trajectory', 'e0')]
 			colnames(e0Fpred) <- c('country_code', 'year', 'trajectory', 'value')
@@ -491,7 +484,7 @@ load.inputs <- function(inputs, start.year, present.year, end.year, wpp.year, ve
 			if(!file.exists(file.name)) 
 				stop('File ', file.name, 
 					' does not exist.\nSet e0M.sim.dir, e0M.file or change WPP year.')
-			if(verbose) cat('\nLoading ', file.name, '\n')
+			if(verbose) cat('\nLoading ', file.name)
 			e0Mpred <- read.csv(file=file.name, comment.char='#', check.names=FALSE)
 			e0Mpred <- e0Mpred[,c('LocID', 'Year', 'Trajectory', 'e0')]
 			colnames(e0Mpred) <- c('country_code', 'year', 'trajectory', 'value')
@@ -508,7 +501,8 @@ load.inputs <- function(inputs, start.year, present.year, end.year, wpp.year, ve
 			} else e0Mpred <- get.e0.prediction(inputs$e0M.sim.dir, mcmc.dir=NA) # independent from female
 		} else
 			e0Mpred <- .load.wpp.traj('e0M', wpp.year)
-	} 
+	}
+	
 	# Get TFR
 	if(!is.null(inputs$tfr.file)) {
 		if(inputs$tfr.file == 'median_')
@@ -534,7 +528,7 @@ load.inputs <- function(inputs, start.year, present.year, end.year, wpp.year, ve
 				'e0Mpred', 'e0Fpred', 'TFRpred', 'migMpred', 'migFpred', 'estim.years', 'proj.years', 'wpp.year', 
 				'start.year', 'present.year', 'end.year', 'observed'))
 		assign(par, get(par), envir=inp)
-	inp$pop.matrix <- list(male=popm.matrix, female=popf.matrix)
+	inp$pop.matrix <- list(male=pop.ini.matrix[['M']], female=pop.ini.matrix[['F']])
 	return(inp)
 }
 
@@ -657,7 +651,8 @@ get.country.inputs <- function(country, inputs, nr.traj, country.name) {
 	}
 	e <- new.env()
 	e$inputs <- inputs
-	for(par in c('migMpred', 'migFpred')) {
+	for(sex in c('M', 'F')) {
+		par <- paste0('mig', sex, 'pred')
 		if(is.null(inputs[[par]])) next
 		inpc[[par]] <- .get.migration.traj(e, par, country)
 		if(is.null(inpc[[par]])) next
@@ -744,7 +739,7 @@ get.country.inputs <- function(country, inputs, nr.traj, country.name) {
 		}		
 		if(par == 'migMpred' && (is.null(inpc[[par]]) || (!is.null(inpc[[par]]) && ncol(inpc[[par]])==1))) {
 			indices[[dependencies[[par]]]] <- rep(1, nr.traj)
-			next # using one trajectory , male is default
+			next # using one trajectory, male is default
 		}
 		traj.available <- ncol(inpc[[par]])
 		if (traj.available == ncol(inpc[[dependencies[[par]]]])) # same number of trajectories, therefore same indices
@@ -757,9 +752,11 @@ get.country.inputs <- function(country, inputs, nr.traj, country.name) {
 		inpc[[par]] <- inpc[[par]][,indices[[par]], drop=FALSE]
 		inpc[[par]] <- inpc[[par]][as.character(inputs$proj.years),, drop=FALSE]
 	}
+	inpc$mig.nr.traj <- 1
 	for(par in c('migMpred', 'migFpred')) { # age-specific, thus 3-d arrays
 		if(is.null(inpc[[par]])) next
 		inpc[[par]] <- inpc[[par]][,indices[[par]], , drop=FALSE]
+		inpc$mig.nr.traj <- length(indices[[par]])
 	}
 	inpc$observed <- obs
 	return(inpc)
@@ -769,7 +766,7 @@ get.traj.index <- function(nr.traj, traj, traj.dim=2, sample=FALSE) {
 	traj.tot <- dim(traj)[traj.dim]
 	if(nr.traj >= traj.tot) 
 		return(if(sample) sample(1:traj.tot, nr.traj, replace=TRUE) else 1:traj.tot)
-	return(if(sample) sample(1:traj.tot, nr.traj) else seq(1, traj.tot, length=nr.traj))
+	return(if(sample) sample(1:traj.tot, nr.traj) else as.integer(seq(1, traj.tot, length=nr.traj)))
 }
 
 
@@ -808,9 +805,10 @@ modifiedLC <- function (npred, mxKan, eopm, eopf, verbose=FALSE, debug=FALSE) {
     #if(debug) print('Start check ===========================')
     #Get the projected kt from eo, and make projection of Mx
     #stop('')
+    nproj <- npred
     for (mxYKan in list(mxKan$female, mxKan$male)) { # iterate over male and female
     	#print(c('sex: ', mxYKan$sex))
-    	res <- .C("LC", as.integer(npred), as.integer(mxYKan$sex), as.numeric(mxYKan$ax), as.numeric(mxKan$bx), 
+    	res <- .C("LC", as.integer(nproj), as.integer(mxYKan$sex), as.numeric(mxYKan$ax), as.numeric(mxKan$bx), 
 			as.numeric(eop[[mxYKan$sex]]), Kl=as.numeric(mxKan$kl[[mxYKan$sex]]), Ku=as.numeric(mxKan$ku[[mxYKan$sex]]), 
 			constrain=as.integer(mxYKan$sex == 1), 
 			#constrain=as.integer(0),
@@ -822,7 +820,6 @@ modifiedLC <- function (npred, mxKan, eopm, eopf, verbose=FALSE, debug=FALSE) {
 		Mx[[mxYKan$sex]] <- matrix(res$Mx, nrow=28)
 		lx[[mxYKan$sex]] <- matrix(res$lx, nrow=28)
     }
-    #stop('')
 	return(list(sr=sr, LLm=LLm, mx=Mx, lx=lx))    
 }
 
@@ -980,12 +977,13 @@ StoPopProj <- function(npred, inputs, LT, asfr, mig.pred=NULL, mig.type=NULL, co
 	totp <- c(sum(popm[,1]+popf[,1]), rep(0, npred))
 	btageM <- btageF <- matrix(0, nrow=7, ncol=npred) # births by age of mother and sex of child
 	deathsM <- deathsF <- matrix(0, nrow=27, ncol=npred)
+	nproj <- npred
 	migM <- as.matrix(if(!is.null(mig.pred[['M']])) mig.pred[['M']] else inputs[['MIGm']])
 	migF <- as.matrix(if(!is.null(mig.pred[['F']])) mig.pred[['F']] else inputs[['MIGf']])
 	#migM <- as.matrix(if(!is.null(mig.pred[['M']])) apply(mig.pred[['M']], 2, '/', popm[1:21,1]) else inputs[['MIGm']])
 	#migF <- as.matrix(if(!is.null(mig.pred[['F']])) apply(mig.pred[['F']], 2, '/', popf[1:21,1]) else inputs[['MIGf']])
 	#stop('')
-	res <- .C("TotalPopProj", npred, as.numeric(migM), as.numeric(migF), nrow(migM), ncol(migM), mig.type,
+	res <- .C("TotalPopProj", as.integer(nproj), as.numeric(migM), as.numeric(migF), nrow(migM), ncol(migM), as.integer(mig.type),
 		srm=LT$sr[[1]], srf=LT$sr[[2]], asfr=as.numeric(as.matrix(asfr)), 
 		srb=as.numeric(as.matrix(inputs$SRB)), popm=popm, popf=popf, totp=totp,
 		Lm=as.numeric(LT$LLm[[1]]), Lf=as.numeric(LT$LLm[[2]]), 
@@ -1054,89 +1052,90 @@ compute.observedVE <- function(inputs, pop.matrix, mig.type, mxKan, country.code
 }
 
 write.pop.projection.summary <- function(pop.pred, what=NULL, expression=NULL, output.dir=NULL, ...) {
-	if (is.null(output.dir)) output.dir <- pop.output.directory(pop.pred)
+	pred <- as.environment(pop.pred) # to avoid lots of copying and allowing attaching cache
+	if (is.null(output.dir)) output.dir <- pop.output.directory(pred)
 	if(!file.exists(output.dir)) dir.create(output.dir, recursive=TRUE)
 	all.what <- c('pop', 'popsex', 'popsexage', 'popage', 'births', 'birthssex', 'birthsage', 'birthssexage', 
 			'deaths', 'deathssex', 'deathsage', 'deathssexage', 'srsexage', 'fertility', 'fertilityage')
 	what <- if(is.null(what)) all.what else match.arg(what, all.what, several.ok=TRUE)
 	params <- list()
 	if(!is.null(expression)) {
+		#write.expression(pop.pred, expression=expression, output.dir=output.dir, ...)
 		what <- 'expression'
 		params <- list(expression=expression)
-	}
-	for(summary.type in what) {
-		if(is.element(summary.type, what))
-			do.call(paste('write.', summary.type, sep=''), c(list(pop.pred, output.dir=output.dir), params, ...))
-	}
+	} #else {
+		for(summary.type in what) 
+			do.call(paste0('write.', summary.type), c(list(pred, output.dir=output.dir), params, ...))
+	#}
 }
 
-write.pop <- function(pop.pred, output.dir) 
-	.write.pop(pop.pred, output.dir=output.dir, bysex=FALSE, byage=FALSE)
+write.pop <- function(pop.pred, output.dir, ...) 
+	.write.pop(pop.pred, output.dir=output.dir, bysex=FALSE, byage=FALSE, ...)
 	
-write.popsex <- function(pop.pred, output.dir) 
-	.write.pop(pop.pred, output.dir=output.dir, bysex=TRUE, byage=FALSE, what.log='population')
+write.popsex <- function(pop.pred, output.dir, ...) 
+	.write.pop(pop.pred, output.dir=output.dir, bysex=TRUE, byage=FALSE, what.log='population', ...)
 	
-write.popsexage <- function(pop.pred, output.dir) 
-	.write.pop(pop.pred, output.dir=output.dir, bysex=TRUE, byage=TRUE, what.log='population')
+write.popsexage <- function(pop.pred, output.dir, ...) 
+	.write.pop(pop.pred, output.dir=output.dir, bysex=TRUE, byage=TRUE, what.log='population', ...)
 	
-write.popage <- function(pop.pred, output.dir) 
-	.write.pop(pop.pred, output.dir=output.dir, bysex=FALSE, byage=TRUE, what.log='population')
+write.popage <- function(pop.pred, output.dir, ...) 
+	.write.pop(pop.pred, output.dir=output.dir, bysex=FALSE, byage=TRUE, what.log='population', ...)
 	
-write.births <- function(pop.pred, output.dir) 
+write.births <- function(pop.pred, output.dir, ...) 
 	.write.pop(pop.pred, output.dir=output.dir, bysex=FALSE, byage=FALSE, vital.event='births', 
-			file.suffix='births', what.log='total births')
+			file.suffix='births', what.log='total births', ...)
 	
-write.birthssex <- function(pop.pred, output.dir) 
+write.birthssex <- function(pop.pred, output.dir, ...) 
 	.write.pop(pop.pred, output.dir=output.dir, bysex=TRUE, byage=FALSE, vital.event='births', 
-			file.suffix='births', what.log='births')
+			file.suffix='births', what.log='births', ...)
 
-write.birthsage <- function(pop.pred, output.dir) 
+write.birthsage <- function(pop.pred, output.dir, ...) 
 	.write.pop(pop.pred, output.dir=output.dir, bysex=FALSE, byage=TRUE, vital.event='births', 
-			file.suffix='births', what.log='births')
+			file.suffix='births', what.log='births', ...)
 
-write.birthssexage <- function(pop.pred, output.dir) 
+write.birthssexage <- function(pop.pred, output.dir, ...) 
 	.write.pop(pop.pred, output.dir=output.dir, bysex=TRUE, byage=TRUE, vital.event='births', 
-			file.suffix='births', what.log='births')
+			file.suffix='births', what.log='births', ...)
 
-write.deaths <- function(pop.pred, output.dir) 
+write.deaths <- function(pop.pred, output.dir, ...) 
 	.write.pop(pop.pred, output.dir=output.dir, bysex=FALSE, byage=FALSE, vital.event='deaths', 
-			file.suffix='deaths', what.log='total deaths')
+			file.suffix='deaths', what.log='total deaths', ...)
 	
-write.deathssex <- function(pop.pred, output.dir) 
+write.deathssex <- function(pop.pred, output.dir, ...) 
 	.write.pop(pop.pred, output.dir=output.dir, bysex=TRUE, byage=FALSE, vital.event='deaths', 
-			file.suffix='deaths', what.log='deaths')
+			file.suffix='deaths', what.log='deaths', ...)
 
-write.deathsage <- function(pop.pred, output.dir) 
+write.deathsage <- function(pop.pred, output.dir, ...) 
 	.write.pop(pop.pred, output.dir=output.dir, bysex=FALSE, byage=TRUE, vital.event='deaths', 
-			file.suffix='deaths', what.log='deaths')
+			file.suffix='deaths', what.log='deaths', ...)
 	
-write.deathssexage <- function(pop.pred, output.dir) 
+write.deathssexage <- function(pop.pred, output.dir, ...) 
 	.write.pop(pop.pred, output.dir=output.dir, bysex=TRUE, byage=TRUE, vital.event='deaths', 
-			file.suffix='deaths', what.log='deaths')
+			file.suffix='deaths', what.log='deaths', ...)
 			
-write.srsexage <- function(pop.pred, output.dir) 
+write.srsexage <- function(pop.pred, output.dir, ...) 
 	.write.pop(pop.pred, output.dir=output.dir, bysex=TRUE, byage=TRUE, vital.event='survival', 
-			file.suffix='sr', what.log='survival ratio')
+			file.suffix='sr', what.log='survival ratio', digits=litem('digits', list(...), 4))
 			
-write.fertility <- function(pop.pred, output.dir) 
+write.fertility <- function(pop.pred, output.dir, ...) 
 	.write.pop(pop.pred, output.dir=output.dir, bysex=FALSE, byage=FALSE, vital.event='fertility', 
-			file.suffix='asfr', what.log='fertility rate')
+			file.suffix='asfr', what.log='fertility rate', digits=litem('digits', list(...), 4))
 			
-write.fertilityage <- function(pop.pred, output.dir) 
+write.fertilityage <- function(pop.pred, output.dir, ...) 
 	.write.pop(pop.pred, output.dir=output.dir, bysex=FALSE, byage=TRUE, vital.event='fertility', 
-			file.suffix='asfr', what.log='fertility rate')	
+			file.suffix='asfr', what.log='fertility rate', digits=litem('digits', list(...), 4))
 	
 write.expression <- function(pop.pred, expression, output.dir, file.suffix='expression', expression.label=expression,  
-								include.observed=FALSE, decimal=NULL) {
+								include.observed=FALSE, digits=NULL, adjust=FALSE, adj.to.file=NULL, end.time.only=FALSE) {
 	cat('Creating summary file for expression ', expression, ' ...\n')
 	header <- list(country.name='country_name',  country.code='country_code', variant='variant')
 	variant.names <- c('median', 'lower 80', 'upper 80', 'lower 95', 'upper 95')
 	nr.var <- length(variant.names)
-	pred.period <- get.pop.prediction.periods(pop.pred)
+	pred.period <- get.pop.prediction.periods(pop.pred, end.time.only=end.time.only)
 	nr.proj <- length(pred.period)
 	nr.obs <- 0
 	if(include.observed) {
-		obs.period <- get.pop.observed.periods(pop.pred)
+		obs.period <- get.pop.observed.periods(pop.pred, end.time.only=end.time.only)
 		nr.obs <- length(obs.period)-1
 		obs.period <- obs.period[-(nr.obs+1)] # remove the last one because the same as the first projection period
 		for(iyear in 1:nr.obs) header[[paste('year', iyear, sep='')]] <- obs.period[iyear]
@@ -1149,12 +1148,13 @@ write.expression <- function(pop.pred, expression, output.dir, file.suffix='expr
 		#copy the same data into the variant rows 
 		result <- matrix(NA, nrow=nrow(res)*5, ncol=ncol(res))
 		for(i in 1:5) result[seq(i,by=5, length=nrow(res)),] <- res
-	}	
+	}
+	if(adjust && is.null(pop.pred$adjust.env)) pop.pred$adjust.env <- new.env()
 	for(iyear in 1:nr.proj) {	
 		result <- cbind(result, as.vector(t(get.pop.from.expression.all.countries(expression, pop.pred, 
-						quantiles=c(0.5, 0.1, 0.9, 0.025, 0.975), projection.index=iyear))))
+						quantiles=c(0.5, 0.1, 0.9, 0.025, 0.975), projection.index=iyear, adjust=adjust, adj.to.file=adj.to.file))))
 	}
-	if(!is.null(decimal)) result <- round(result, decimal)
+	if(!is.null(digits)) result <- round(result, digits)
 	colnames(result) <- col.names
 	result <- cbind(country.name=rep(as.character(pop.pred$countries$name), each=nr.var), 
 					country.code=rep(pop.pred$countries$code, each=nr.var),
@@ -1171,7 +1171,7 @@ write.expression <- function(pop.pred, expression, output.dir, file.suffix='expr
 }	
 	
 .write.pop <- function(pop.pred, output.dir, bysex=FALSE, byage=FALSE, vital.event=NULL, file.suffix='tpop', 
-							what.log='total population') {
+							what.log='total population', digits=0, adjust=FALSE) {
 	cat('Creating summary file of ', what.log, ' ')
 	if(bysex) cat('by sex ')
 	if(byage) cat('by age ')
@@ -1181,8 +1181,8 @@ write.expression <- function(pop.pred, expression, output.dir, file.suffix='expr
 	if(byage) header[['age']] <- 'age'
 	variant.names <- c('median', 'lower 80', 'upper 80', 'lower 95', 'upper 95')
 	nr.var <- length(variant.names)
-	pred.period <- get.pop.prediction.periods(pop.pred)
-	if(!is.null(vital.event)) pred.period <- pred.period[2:length(pred.period)]
+	pred.period <- get.pop.prediction.periods(pop.pred, end.time.only=is.null(vital.event))
+	#if(!is.null(vital.event)) pred.period <- pred.period[2:length(pred.period)]
 	nr.proj <- length(pred.period)
 	for (i in 1:nr.proj) 
 		header[[paste('year', i, sep='')]] <- pred.period[i]
@@ -1193,6 +1193,7 @@ write.expression <- function(pop.pred, expression, output.dir, file.suffix='expr
 	age.index <- c(TRUE, rep(FALSE, length(pop.pred$ages)))
 	if(byage) age.index <- !age.index
 	ages <- 1:length(pop.pred$ages)
+	if(adjust && is.null(pop.pred$adjust.env)) pop.pred$adjust.env <- new.env()
 	for (country in 1:nrow(pop.pred$countries)) {
 		country.obj <- get.country.object(country, country.table=pop.pred$countries, index=TRUE)
 		for(sex in c('both', 'male', 'female')[sex.index]) {
@@ -1223,7 +1224,7 @@ write.expression <- function(pop.pred, expression, output.dir, file.suffix='expr
 					this.result <- cbind(this.result, age=rep(get.age.labels(pop.pred$ages)[age], nr.var))
 				}
 				if(is.null(vital.event)) {
-					quant <- get.pop.trajectories(pop.pred, country.obj$code, nr.traj=0, sex=sex, age=age)$quantiles
+					quant <- get.pop.trajectories(pop.pred, country.obj$code, nr.traj=0, sex=sex, age=age, adjust=adjust)$quantiles
 					traj <- NULL
 					reload <- TRUE
 				} else { # vital event
@@ -1242,7 +1243,7 @@ write.expression <- function(pop.pred, expression, output.dir, file.suffix='expr
 											trajectories=traj, reload=reload),
 					get.pop.traj.quantiles(quant, pop.pred, country.obj$index, country.obj$code, pi=95, 
 											trajectories=traj, reload=reload)),
-					0)
+					digits)
 				colnames(proj.result) <- col.names
 				this.result <- cbind(this.result, proj.result)
 				result <- rbind(result, this.result)
@@ -1322,4 +1323,12 @@ unblock.gtk.if.needed <- function(status, gui.opts=list()) {
 	}
 }
 
-unblock.gtk <- function(...) bayesTFR:::unblock.gtk(...)
\ No newline at end of file
+unblock.gtk <- function(...) bayesTFR:::unblock.gtk(...)
+
+create.pop.cluster <- function(nr.nodes, ...) {
+	cl <- makeCluster(nr.nodes, ...)
+	lib.paths <- .libPaths()
+	clusterExport(cl, c("lib.paths"), envir=environment())
+	clusterEvalQ(cl, {.libPaths(lib.paths); library(bayesPop)})
+	return(cl)
+}
\ No newline at end of file
