@@ -113,7 +113,8 @@ void LCEoKtC(int sex, double *ax, double *bx,
 	LifeTableC(sex, 27, mxm, LTl, lm);
 	
 	if(eop < sum(LTl, dim)) {
-		LLm = LTl;
+		for (i=0; i < dim; ++i) LLm[i]=LTl[i];
+		for (i=0; i < 28; ++i) Mx[i] = mxm[i];
 		return;
 	}
 	for (i=0; i < 28; ++i) {
@@ -122,7 +123,8 @@ void LCEoKtC(int sex, double *ax, double *bx,
 	LifeTableC(sex, 27, mxm, LTu, lm);
 
 	if(eop > sum(LTu, dim)) {
-		LLm = LTu;
+		for (i=0; i < dim; ++i) LLm[i]=LTu[i]; 
+		for (i=0; i < 28; ++i) Mx[i] = mxm[i];
 		if(debug==1) Rprintf("\nBreturn %f", sum(LTu, dim));
 		return;
 	}
@@ -362,7 +364,6 @@ void TotalPopProj(int *npred, double *MIGm, double *MIGf, int *migr, int *migc,
 				break;
 		}
 	}
-
 	/* Population projection for one trajectory */
 	for(j=1; j<(n+1); ++j) {
 		jve = j-1;
